package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/** 
 * Abstrakte Elternklasse aller zu parsenden Elemente
 * des Domain Specific Language Parsers
 * 
 * noch offen: getClone
 * 
 * vielleicht mal auftrennen in zwei Oberklassen fr terminale und nicht terminale Symbole
 */
public abstract class AbstractElement
{

    // ---------- abstrakte Deklarationen -----------------------------------

    /** 
     * jedes Element muss eine Lnge in Zeichen haben
     * 
     * @return Lnge
     */
    public abstract int getLength();

    /** 
     * toString mit Einrckung
     * 
     * @param indent Einrcktiefe
     * @return String-Darstellung
     */
    public abstract String toString(int indent);

    /** 
     * ob das Element terminal (einfach) oder nicht terminal (hat Unterelemente) ist
     * 
     * @return ob terminal (ja/nein)
     */
    public abstract boolean isTerminal();

    /** 
     * textliche Darstellung, nur bei Terminalsymbolen
     * 
     * @return String mit Inhalt
     */
    public abstract String getContent();

    /** 
     * textliche Darstellung ohne eventuelle Begrenzungszeichen (Klammern oder hnlich) nur bei Terminalsymbolen
     * 
     * @return textliche Darstellung ohne eventuelle Begrenzungszeichen 
     *     (Klammern oder hnlich)
     */
    public abstract String getNettoContent();

    /** 
     * ob leer, nur bei nicht Terminalsymbolen
     * 
     * @return ob leer (ja/nein)
     */
    public abstract boolean isEmpty();

    /** 
     * Ende-Position des Elementes,
     * nur bei nicht Terminalsymbolen
     * 
     * @return Anfangs-Position
     */
    public abstract SourcePosition getEndSourcePos();

    /** ob sich das Element ber mehrere Zeilen erstreckt
     * 
     * @return ob multilined (ja/nein)
     */
    public abstract boolean isMultiLined();

    /** 
     * Kommentare Lschen
     * 
     * @param pAllComments ob alle Kommentare gelscht werden soll,
     *                     oder nur Kommentare die kein Javadoc sind
     */
    public abstract void stripComments(final boolean pAllComments);

    // ----------------------- Members und Methods -------------------------------------

    /** 
     * Position des Elementes im Sourcecode der Unit (Datei)
     */
    protected SourcePosition srcPos;
    
    /** 
     * Anfangs-Position des Elementes
     * 
     * @return Anfangs-Position
     */
    public final SourcePosition getSourcePos()
    {
        return this.srcPos.getClone();
    }

    // --------------------- Util-Methoden ---------------------------------------------



    /** 
     * Hilfsmethode fr alle Element-toString()-Methoden
     * 
     * @param indent Einrcktiefe
     * @return String fr toString
     */
    protected StringBuffer toStringHelp(int indent)
    {
        StringBuffer strBuff = new StringBuffer();

        strBuff.append(StringUtil.spaces(indent));
        strBuff.append(StringUtil.getShortClassName(this));
        strBuff.append(" " + " pos: " + getSourcePos().getPos() + " row: " + getSourcePos().getRow()
                + " col: " + getSourcePos().getCol() + " length: " + getLength() + "\n" );
        if ( this instanceof Segment ) // ElementList ) // CommaSeparatedList ) // ParenthesisArea ) // AbstractNonTerminalElement )
        {
            strBuff.append( ( (AbstractNonTerminalElement) this ).getNettoContent() + "\n" );
        }
        return strBuff;
    }

		  /** 
		   * statische Parse-Methode
		   * 
		   * @exception DomainParseException wenn nicht erfolgreich geparst
		   * @param pStr Quell-String gesamte Unit (Datei)
		   * @param pSrcPos Position des zu parsenden Elementes
		   * @param pParserContext Einstellungen des Parsers
		   * @return geparstes Element
		   */
  
    /* package */ static AbstractElement parseElement(
            final String pStr,
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        // System.out.println( "Element parseElement start" );

        AbstractElement element = null;

        // Element parsen
        if (element == null)
        {
            element = InlineComment.parse(pStr, pSrcPos);
        }
        if (element == null)
        {
            element = BracketComment.parse(pStr, pSrcPos);
        }
        if (element == null)
        {
            element = Number.parse(pStr, pSrcPos);
        }
//        if (element == null)
//        {
//            element = ComplexIdentifier.parse(pStr, pSrcPos, pParserContext);
//        }
        if (element == null)
        {
            element = Identifier.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            element = Operator.parse(pStr, pSrcPos);
        }
        if (element == null)
        {
            element = ParenthesisArea.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            element = BracketArea.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            element = QuoteStringLiteral.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            element = HighCommaStringLiteral.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            element = BracesArea.parse(pStr, pSrcPos, pParserContext);
        }
        if (element == null)
        {
            throw new DomainParseException("no element parsed", pSrcPos);
        }

        // System.out.println( "Element parseElement stop: " + element );

        return element;
    }

}
