package de.cnc.domainparser;


import java.util.ArrayList;
import java.util.Iterator;

import de.cnc.domainparser.util.DomainParserUtil;
import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Ein Segment entspricht einer Deklaration (package, import, variable) 
 * oder Klasse, Methode . <br>
 * <br/>
 * Ein Segment wird entweder durch ein Semikolon oder ein {@link de.cnc.domainparser.BracesArea}
 * abgeschlossen.<br/>
 * <br/>
 * Segment ;
 * <br><br>
 * oder
 * <br><br>
 * ... { ... }
 * <br><br>
 */
public class Segment extends AbstractNonTerminalElement
{
    /** 
     * ob mit curly braces <b>{}</b> oder
     * mit Semikolon <b>;</b> endend.
     */
    private boolean isBracesSegment = false;

    /** Liste der Elemente */
    private ArrayList elementList = new ArrayList();

    /** kompletter Source dieses Segmentes als String */
    private final String completeSrcStr;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private Segment(SourcePosition pSrcPos, String pSrcStr)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();

        // gesamten Quelltext referenzieren
        this.completeSrcStr = pSrcStr;
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static Segment parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        // System.out.println( "Segment#parse pStr.charAt( pSrcPos.getPos() ): " + pStr.charAt( pSrcPos.getPos() ) );
        Segment segment = new Segment(pSrcPos, pStr);

        // Vermerken der Anfangsposition
        int anfPos = pSrcPos.getPos();        

        char runChar = pStr.charAt(pSrcPos.getPos());

        if (runChar == ';')
        {
            // Lnge vermerken
            segment.length = pSrcPos.getPos() - anfPos;
        
            segment.endSourcePosition = pSrcPos.getClone();

            // gleich am Anfang steht ein Semikolon
            pSrcPos.addOne(); // Semikolon berspringen

            return segment;
        }

        while (pSrcPos.getPos() < pStr.length())
        {
            AbstractElement element = AbstractElement.parseElement(pStr, pSrcPos, pParserContext);

            // System.out.println( "Segment#element " + element );

            // Element vermerken
            segment.elementList.add(element);

            // Lnge vermerken
            segment.length = pSrcPos.getPos() - anfPos;
        
            segment.endSourcePosition = pSrcPos.getClone();

            // nach dem Element folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

            if (element instanceof BracesArea)
            {
                // ein BracesArea (geschwungene Klammern) beendet ein Segment
                segment.isBracesSegment = true;
                break;
            }

            if (pSrcPos.getPos() < pStr.length())
            {
                runChar = pStr.charAt(pSrcPos.getPos());

                if (runChar == ';')
                {
                    pSrcPos.addOne(); // Semikolon berspringen
                    break;
                }
                if (runChar == '}' || runChar == ')' || runChar == ']')
                {
                    // scheinbar Ende eines Bereiches
                    break;
                }
            }
        }
        
        return segment;
    }

    public boolean isBracesSegment()
    {
        return this.isBracesSegment;
    }

    /**
     * 
     * @return
     */
    public int elementCount()
    {
        return this.elementList.size();
    }

    /**
     * 
     * @param pI
     * @return
     */
    public AbstractElement getElement(int pI)
    {
        return (AbstractElement) this.elementList.get(pI);
    }

    /**
     * 
     */
    public String toString()
    {
        return toString(0);
    }

    /**
     * 
     * @param indent
     * @return
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        for (int i = 0; i < this.elementList.size(); i++)
        {
            strBuff.append(((AbstractElement) this.elementList.get(i)).toString(indent + 1) + "\n");
        }
        if (this.elementList.size() < 1)
        {
            strBuff.append(StringUtil.spaces(indent + 1) + "segment is empty" + "\n");
        }
        return strBuff.toString();
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return this.elementList.isEmpty();
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        // throw new UnsupportedOperationException();
        return this.completeSrcStr.substring(this.srcPos.getPos(), this.srcPos.getPos() + this.length + 1 );
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        // throw new UnsupportedOperationException();
        return this.completeSrcStr.substring(this.srcPos.getPos(), this.srcPos.getPos() + this.length);
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Lschen Kommentare
     */
    public void stripComments(boolean pAllComments)
    {
        for (Iterator iter = this.elementList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            if (o instanceof InlineComment)
            {
                iter.remove();
            }
            else if (o instanceof BracketComment 
                && ( pAllComments || ! ( (BracketComment) o ).isJavadoc() ) )
            {
                iter.remove();
            }
            else if ( o instanceof AbstractNonTerminalElement )
            {
                ((AbstractNonTerminalElement) o).stripComments(pAllComments);
            }
        }
    }

    /**
     * Lschen leere Sub-Segmente
     */
    public void stripEmptySegments()
    {
        for (Iterator iter = this.elementList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            if (o instanceof BracesArea)
            {
                ( (BracesArea) o ).stripEmptySegments();
            }
        }
    }
}
